import 'package:equatable/equatable.dart';
import 'package:flutter/material.dart';
import 'package:json_annotation/json_annotation.dart';
import 'package:ocg_location/ocg_location.dart';
import 'package:ocg_profile/ocg_profile.dart';

part 'story.g.dart';

class StoryType {
  static String get image => "image";
  static String get video => "video";
  static String get quiz => "quiz";
  static String get quizList => "quizList";
}

@JsonSerializable(explicitToJson: true)
class Story extends Equatable {
  final String id;
  final Profile profile;
  final DateTime recordedAt;
  final StoryBackground background;
  final StoryData data;
  final List<Profile> seenBy;
  final bool isSeen;
  final bool isSelf;
  final Address address;

  Story(this.id, this.profile, this.recordedAt, this.background, this.data,
      this.seenBy, this.isSeen, this.isSelf, this.address);

  Story.named(
      {this.id,
      this.profile,
      this.recordedAt,
      this.background,
      this.data,
      this.seenBy,
      this.isSeen,
      this.isSelf,
      this.address});

  Story copyWith(
      {String id,
      StoryData data,
      StoryBackground background,
      DateTime recordedAt,
      List<Profile> seenBy,
      Profile profile,
      bool isSeen,
      bool isSelf,
      MetaData metaData}) {
    return Story.named(
        id: id ?? this.id,
        data: data ?? this.data,
        profile: profile ?? this.profile,
        background: background ?? this.background,
        recordedAt: recordedAt ?? this.recordedAt,
        seenBy: seenBy ?? this.seenBy,
        isSeen: isSeen ?? this.isSeen,
        isSelf: isSelf ?? this.isSelf,
        address: address ?? this.address);
  }

  @override
  List<Object> get props => [id];

  String get thumbnail {
    if (background is StoryImage) {
      return (background as StoryImage).url;
    }
    if (background is StoryVideo) {
      return (background as StoryVideo).thumbnail;
    }
    return null;
  }

  Map<String, dynamic> toJson() => _$StoryToJson(this);

  factory Story.fromJson(json) {
    var story = _$StoryFromJson(json);
    var background = json["background"] != null
        ? json["background"]["type"] == StoryType.image
            ? StoryImage.fromJson(json["background"])
            : json["background"]["type"] == StoryType.video
                ? StoryVideo.fromJson(json["background"])
                : null
        : null;
    return story.copyWith(background: background);
  }
}

@JsonSerializable(explicitToJson: true)
class StoryUser {
  final String id;
  final String name;
  final String profilePic;

  StoryUser(this.id, this.name, this.profilePic);
  StoryUser.named({this.id, this.name, this.profilePic});

  Map<String, dynamic> toJson() => _$StoryUserToJson(this);

  factory StoryUser.fromJson(json) => _$StoryUserFromJson(json);
}

@JsonSerializable(explicitToJson: true)
class StoryBackground {
  final String type;

  StoryBackground(
    this.type,
  );

  Map<String, dynamic> toJson() => _$StoryBackgroundToJson(this);

  factory StoryBackground.fromJson(json) => _$StoryBackgroundFromJson(json);
}

@JsonSerializable(explicitToJson: true)
class StoryVideo extends StoryBackground {
  final String url;
  final String thumbnail;

  StoryVideo(this.url, this.thumbnail) : super(StoryType.video);

  StoryVideo.named({this.url, this.thumbnail}) : super(StoryType.video);

  Map<String, dynamic> toJson() {
    var json = _$StoryVideoToJson(this);
    json["type"] = StoryType.video;
    return json;
  }

  factory StoryVideo.fromJson(json) => _$StoryVideoFromJson(json);
}

@JsonSerializable(explicitToJson: true)
class StoryImage extends StoryBackground {
  final String url;

  StoryImage(this.url) : super(StoryType.image);

  StoryImage.named({this.url}) : super(StoryType.image);

  Map<String, dynamic> toJson() {
    var json = _$StoryImageToJson(this);
    json["type"] = StoryType.image;
    return json;
  }

  factory StoryImage.fromJson(json) => _$StoryImageFromJson(json);
}

@JsonSerializable(explicitToJson: true)
class StoryData {
  final String type;

  StoryData(
    this.type,
  );

  Map<String, dynamic> toJson() {
    var json = _$StoryDataToJson(this);
    return json;
  }

  factory StoryData.fromJson(json) => _$StoryDataFromJson(json);
}

///
/// Story Collection
///

class ProfileStories {
  final Profile profile;
  final List<Story> stories;
  final bool isSelf;

  ProfileStories(this.profile, this.stories, this.isSelf);

  ProfileStories.named({this.profile, this.stories, this.isSelf});

  ProfileStories copyWith({Profile profile, List<Story> stories, bool isSelf}) {
    return ProfileStories.named(
        profile: profile ?? this.profile,
        stories: stories ?? this.stories,
        isSelf: isSelf ?? this.isSelf);
  }

  bool get isSeen {
    bool hasSeen = false;
    stories.forEach((element) {
      hasSeen = hasSeen || (element.isSeen ?? false);
    });
    return hasSeen;
  }
}
