import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/material.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_location/ocg_location.dart';
import 'package:ocg_profile/ocg_profile.dart';
import 'package:ocg_story/ocg_story.dart';
import 'package:ocg_story/src/socket/data.dart';

class StoryFirebaseRepository extends StoryRepository {
  final FirebaseFirestore _firestore;
  final IDHelper idHelper;
  final ProfileRepository profileRepository;

  static const String stories = 'stories';

  CollectionReference get storyCollection => _firestore.collection(stories);

  StoryFirebaseRepository(this.idHelper, this.profileRepository)
      : this._firestore = FirebaseFirestore.instance;

  @override
  Future<AppResponse<bool>> addStorySeen(String id) async {
    try {
      AppResponse<String> profileResponse =
          await profileRepository.getCurrentId();
      if (profileResponse.isSuccess) {
        String profileId = profileResponse.data;
        AppResponse<Story> storyResponse = await read(id);
        if (storyResponse.isSuccess) {
          Story story = storyResponse.data;
          if (story.profile?.id != profileId &&
              !story.seenBy.map((e) => e.id).toList().contains(profileId)) {
            await storyCollection
                .doc("$id")
                .update({"seenBy.$profileId": true});
            return AppResponse.named(data: true);
          } else {
            return AppResponse.named(error: "You can not see you story again");
          }
        } else {
          return AppResponse.named(error: "Story not found");
        }
      }
      return AppResponse.named(error: "Not logged in");
    } catch (e) {
      return AppResponse.named(error: "$e");
    }
  }

  @override
  Future<AppResponse<String>> create(Story object) async {
    try {
      if (object.background == null) {
        return AppResponse.named(error: "Background can be empty");
      }
      AppResponse<String> profileResponse =
          await profileRepository.getCurrentId();
      if (profileResponse.isSuccess) {
        String userId = profileResponse.data;
        String id = idHelper.generateId();
        object = object.copyWith(
            id: id,
            profile: Profile.named(id: userId),
            recordedAt: DateTime.now());
        await storyCollection.doc(id).set(_toJson(object));
        return AppResponse.named(data: id);
      } else {
        return AppResponse.named(error: "Not logged in");
      }
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  Map<String, dynamic> _toJson(Story story) {
    var json = story?.toJson();
    json["profileId"] = story?.profile?.id;
    json["profile"] = null;
    json["seenBy"] = {};
    (story.seenBy ?? []).map((e) => e.id).forEach((element) {
      json["seenBy"][element] = true;
    });
    return json;
  }

  Story _fromJson(json) {
    return Story.fromJson(json);
  }

  Future<Story> _parseJson(storyJson) async {
    var newJson = storyJson ?? {};
    var currentResponse = await profileRepository.getCurrentId();
    if (currentResponse.isSuccess) {
      newJson["isSelf"] = currentResponse.data == storyJson["profileId"];
    }
    if (storyJson["profileId"] is String) {
      var profileResponse =
          await profileRepository.read(storyJson['profileId']);
      if (profileResponse.isSuccess) {
        newJson["profile"] = profileResponse.data.toJson();
      }
    }
    var seenBy = newJson["seenBy"];
    // TODO fetch only for current user
    if (seenBy != null && seenBy is Map) {
      var ids = [];
      seenBy.forEach((key, value) {
        ids.add(key);
      });
      var result = await Future.wait(ids.map((e) async {
        var response = await profileRepository.read(e);
        if (response.isSuccess) {
          return response.data;
        }
      }).toList());
      result = result.where((element) => element != null).toList();
      newJson["seenBy"] =
          List<Profile>.from(result).map((e) => e.toJson()).toList();
    } else {
      newJson["seenBy"] = [];
    }
    Story story = _fromJson(newJson);
    if (story.seenBy
            .indexWhere((element) => element.id == currentResponse.data) >=
        -1) {
      story = story.copyWith(isSeen: true);
    }
    return story;
  }

  @override
  Future<AppResponse<bool>> delete(String id) async {
    try {
      var response = await read(id);
      if (response.isSuccess) {
        if (response.data.isSelf ?? false) {
          await storyCollection.doc(id).delete();
        } else {
          return AppResponse.named(error: "User can not delete other story");
        }
      }
      return AppResponse.named(data: true);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<List<Story>>> getList(
      {int page = 1, int count = 10}) async {
    var snapshot = await storyCollection.limit(count * page).get();
    var documents =
        snapshot.docs.map((e) => e.data()).skip((page - 1) * count).toList();
    var list = (await Future.wait(documents.map((e) async {
      return await _parseJson(e);
    }).toList()))
        .where((element) => element != null)
        .toList();
    return AppResponse.named(data: list);
  }

  @override
  Future<AppResponse<Story>> read(String id) async {
    try {
      var document = await storyCollection.doc(id).get();
      var data = document.data();
      return AppResponse.named(data: await _parseJson(data));
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<bool>> update(Story object) async {
    try {
      await storyCollection.doc(object.id).update(object.toJson());
      return AppResponse.named(data: true);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<List<ProfileStories>>> getFollowing({
    int count = 10,
    int page = 1,
  }) async {
    var query = storyCollection;
    var appResponse = await _getFollowing(query);
    if (appResponse.isSuccess) {
      var stories = appResponse.data;
      return AppResponse.named(data: stories.skip((page - 1) * count).toList());
    } else {
      return AppResponse.named(error: appResponse.error);
    }
  }

  @override
  Future<AppResponse<List<ProfileStories>>> getNearby(
    Position position, {
    int count = 10,
    int page = 1,
  }) async {
    try {
      Profile currentProfile;
      AppResponse<Profile> profileResponse =
          await profileRepository.getCurrent();
      currentProfile = profileResponse.data;
      RangeGeoHash rangeGeoHash = LocationRepository.getGeoHashRange(
          position.latitude, position.longitude,
          distanceInMiles: 10);
      var snapshot = await storyCollection
          .orderBy("address.geohash")
          .limit(count * page)
          .where(
            "address.geohash",
            isGreaterThanOrEqualTo: rangeGeoHash.lower,
          )
          .where("address.geohash", isLessThanOrEqualTo: rangeGeoHash.upper)
          .get();

      var documents = snapshot.docs.map((e) => e.data()).toList();
      documents = documents.skip((page - 1) * count).toList();
      var list =
          await Future.wait(List<Future<Story>>.from(documents.map((e) async {
        return await _parseJson(e);
      }).toList()));
      list = list.where((element) => element != null).toList();
      list = list
          .map((e) => e.copyWith(isSelf: currentProfile?.id == e?.profile?.id))
          .toList();
      List<ProfileStories> profileStories = [];
      Map<Profile, List<Story>> stories = {};
      list.forEach((element) {
        if (stories[element.profile] == null) {
          stories[element.profile] = [];
        }
        stories[element.profile].add(element);
      });
      stories.forEach((key, value) {
        profileStories.add(ProfileStories.named(profile: key, stories: value));
      });
      return AppResponse.named(data: profileStories);
    } catch (e) {
      return AppResponse.named(error: "$e");
    }
  }

  Future<AppResponse<List<ProfileStories>>> _getFollowing(var query) async {
    AppResponse<String> profileResponse =
        await profileRepository.getCurrentId();
    if (profileResponse.isSuccess) {
      String currentId = profileResponse.data;
      List<String> following = [];
      following.add(currentId);
      var followingResponse =
          await profileRepository.getFollowersFollowing(currentId);
      if (followingResponse.isSuccess) {
        var result = List<String>.from(followingResponse.data
            .where((element) => element.from == currentId)
            .map((e) => e.to)
            .toList());
        following.addAll(result);
      }
      var snapshot = await query.where("profileId", whereIn: following).get();
      var documents = snapshot.docs.map((e) => e.data()).toList();
      var list =
          await Future.wait(List<Future<Story>>.from(documents.map((e) async {
        return await _parseJson(e);
      }).toList()));
      list = list
          .map((e) => e.copyWith(isSelf: currentId == e?.profile?.id))
          .toList();
      return AppResponse.named(data: getProfileStories(list));
    } else {
      return AppResponse.named(error: profileResponse.error);
    }
  }

  List<ProfileStories> getProfileStories(List<Story> list) {
    list = list.where((element) => element != null).toList();
    List<ProfileStories> profileStories = [];
    Map<Profile, List<Story>> stories = {};
    list.forEach((element) {
      if (stories[element.profile] == null) {
        stories[element.profile] = [];
      }
      stories[element.profile].add(element);
    });
    stories.forEach((key, value) {
      profileStories.add(ProfileStories.named(
          profile: key, stories: value, isSelf: value[0].isSelf));
    });
    return profileStories;
  }

  @override
  // TODO: implement changes
  Stream<StoryChange> get changes => throw UnimplementedError();
}
