import 'dart:async';
import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_location/ocg_location.dart';
import 'package:ocg_profile/ocg_profile.dart';
import 'package:ocg_socket/ocg_socket.dart';
import 'package:ocg_story/src/models/story.dart';
import 'package:http/http.dart';
import 'package:ocg_story/src/repository/story/base.dart';
import 'package:ocg_story/src/socket/data.dart';
import 'package:rxdart/rxdart.dart';

class StoryNetworkRepository extends StoryRepository {
  final Client client;
  final ProfileRepository profileRepository;
  final BaseSocket baseSocket;
  BehaviorSubject<StoryChange> _subject;

  StoryNetworkRepository(
      {this.client, this.profileRepository, this.baseSocket}) {
    _subject = BehaviorSubject<StoryChange>();
    _listenToChanges();
  }

  Future<void> _listenToChanges() async {
    if (baseSocket == null) {
      return;
    }
    baseSocket.stream.forEach((SocketData element) {
      if (element.type == "STORY_CREATED" && element.data != null)
        _subject.add(StoryCreateChange(element.data["story_id"]));
      else if (element.type == "STORY_DELETED" && element.data != null) {
        _subject.add(StoryDeleteChange(element.data["story_id"]));
      }
    });
  }

  @override
  Future<AppResponse<bool>> addStorySeen(String id) async {
    try {
      String url = '/stories/$id';
      debugPrint("Seen");
      await client.post(
        url,
      );
      return AppResponse.named(data: true);
    } catch (e) {
      debugPrint("Error $e");
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<String>> create(Story story) async {
    try {
      String url = '/stories';
      var json = toJson(story);
      await client.post(url,
          body: jsonEncode(json), headers: ApiConfig.headers);
      return AppResponse.named(data: "id");
    } catch (e) {
      debugPrint("Error $e");
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<bool>> delete(String storyId) async {
    try {
      String url = '/stories/$storyId';
      var response = await client.delete(
        url,
      );
      debugPrint("Delete ${response.body}");
      return AppResponse.named(data: true);
    } catch (e) {
      debugPrint("Error $e");
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<List<Story>>> getList(
      {int count = 10, int page = 1}) async {
    try {
      AppResponse<String> userIdResponse =
          await profileRepository.getCurrentId();
      if (!userIdResponse.isSuccess) {
        return AppResponse.named(error: userIdResponse.error);
      }
      String currentId = userIdResponse.data;
      String url = '/stories';
      var response = await client.get(
        url + "?count=$count",
      );
      var json = jsonDecode(response.body);
      var list = (json["stories"] ?? []) as List;
      var stories = List<Story>.from(await Future.wait((list).map((e) async {
        Story story = await _parseJson(e);
        if (story != null) {
          List<Profile> seenBy = (story.seenBy ?? []).toList();
          story = story.copyWith(
              seenBy: seenBy,
              isSeen: (story.seenBy ?? [])
                      .indexWhere((element) => element.id == currentId) !=
                  -1,
              isSelf: currentId == story.profile?.id);
          return story;
        }
      })))
          .toList();

      return AppResponse.named(
          data: List<Story>.from(
              stories.where((element) => element != null).toList()));
    } catch (e) {
      debugPrint("Error $e");
      return AppResponse.named(error: e.toString());
    }
  }

  Future<Story> _parseJson(Map<String, dynamic> json) async {
    json = cleanJson(json);
    json["id"] = json["id"];
    var profileId = json["author"];
    var profileResponse = await profileRepository.read(profileId);
    if (!profileResponse.isSuccess) {
      return null;
    }
    List<String> seenBy = List<String>.from(json["seenBy"] ?? []);
    var profiles = List<Profile>.from(await Future.wait(seenBy.map((e) async {
      var response = await profileRepository.read(e);
      if (response.isSuccess) {
        return response.data;
      }
    })));
    profiles = profiles.where((element) => element != null).toList();
    json["seenBy"] = [];
    Story story = fromJson(json);
    story = story.copyWith(seenBy: profiles, profile: profileResponse.data);
    return story;
  }

  Map<String, dynamic> cleanJson(Map<String, dynamic> temp) {
    var json = Map<String, dynamic>.from(temp ?? {});
    json["createdAt"] = json["created_at"];
    json["seenBy"] = json["seen_by"];
    json["background"] = (json["misc"] ?? {})["background"] ?? {};
    json["data"] = (json["misc"] ?? {})["data"] ?? {};
    return json;
  }

  Map<String, dynamic> toJson(Story story) {
    return {
      "id": story.id,
      "misc": {
        "background": story.background is StoryImage
            ? (story.background as StoryImage).toJson()
            : story.background is StoryVideo
                ? (story.background as StoryVideo).toJson()
                : {}
      },
      'createdAt': story.recordedAt?.toIso8601String(),
      'profileId': story.profile?.id,
      'seenBy': (story.seenBy ?? []).map((e) => e.toJson()).toList()
    };
  }

  Story fromJson(Map<String, dynamic> json) {
    json = cleanJson(json);
    var background = json["background"] ?? {};
    return Story.named(
        id: json["id"],
        background: background != null
            ? background["type"] == StoryType.image
                ? StoryImage.fromJson(background)
                : background["type"] == StoryType.video
                    ? StoryVideo.fromJson(background)
                    : null
            : null,
        recordedAt: DateTime.tryParse("${json["createdAt"]}"),
        seenBy: List<Profile>.from(((json["seenBy"] ?? []) as List)
            .map((e) => Profile.fromJson(e))
            .toList()),
        profile:
            json["profile"] != null ? Profile.fromJson(json["profile"]) : null,
        data: null);
  }

  @override
  Future<AppResponse<Story>> read(String id) {
    // TODO: implement read
    throw UnimplementedError();
  }

  @override
  Future<AppResponse<bool>> update(Story object) {
    // TODO: implement update
    throw UnimplementedError();
  }

  @override
  Future<AppResponse<List<ProfileStories>>> getFollowing(
      {int count = 10, int page = 1}) async {
    try {
      var response = await getList();
      if (response.isSuccess) {
        List<Story> stories = response.data;
        List<ProfileStories> profileStories = [];
        Map<Profile, List<Story>> storiesMap = {};
        stories.forEach((element) {
          if (storiesMap[element.profile] == null) {
            storiesMap[element.profile] = [];
          }
          storiesMap[element.profile].add(element);
        });
        storiesMap.forEach((key, value) {
          value.sort((a, b) => a.recordedAt.compareTo(b.recordedAt));
          profileStories
              .add(ProfileStories.named(profile: key, stories: value));
        });
        return AppResponse.named(data: profileStories);
      } else {
        return AppResponse.named(error: response.error);
      }
    } catch (e) {
      return AppResponse.named(error: "$e");
    }
  }

  @override
  Future<AppResponse<List<ProfileStories>>> getNearby(Position position,
      {int count = 10, int page = 1}) {
    // TODO: implement getNearby
    throw UnimplementedError();
  }

  @override
  Stream<StoryChange> get changes => _subject.stream;
}
