import 'dart:io';
import 'dart:typed_data';
import 'dart:ui' as ui;
import 'dart:ui';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_story/src/models/basic.dart';
import 'package:ocg_story/src/widgets/moving_widget.dart';
import 'package:path_provider/path_provider.dart';

class StoryCreateWidget extends StatefulWidget {
  final List<StoryBasic> children;
  final Offset deleteOffset;
  final bool isAdvanced;
  final Function(StoryCreateController) onCompleted;
  final Function(StoryBasic basic) onBasicTap;
  final bool canRemoveMoving;

  const StoryCreateWidget({
    Key key,
    @required this.children,
    @required this.deleteOffset,
    @required this.onCompleted,
    this.isAdvanced = false,
    this.onBasicTap,
    this.canRemoveMoving = false,
  }) : super(key: key);

  @override
  State<StatefulWidget> createState() => _StoryCreateWidgetState();
}

class _StoryCreateWidgetState extends State<StoryCreateWidget> {
  @override
  void initState() {
    super.initState();
    WidgetsBinding.instance.addPostFrameCallback((val) {
      widget.onCompleted(StoryCreateController._(this));
    });
  }

  GlobalKey _globalKey = new GlobalKey();
  int _currentRemovable = -1;

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: RepaintBoundary(
        key: _globalKey,
        child: Container(
          color: Colors.white12,
          child: Stack(
            fit: StackFit.expand,
            children: children,
          ),
        ),
      ),
      floatingActionButton: DragTarget<int>(
        onAcceptWithDetails: (details) {
          if (details.data == 1) {}
        },
        onAccept: (data) {
          debugPrint("Data $data");
          if (data >= 0) {
            widget.children.removeAt(data);
          }
          setState(() {});
        },
        onMove: (details) {
          // if (details.data != -1 && _currentRemovable < 0) {
          //   setState(() {
          //     _currentRemovable = details.data;
          //   });
          // }
        },
        onLeave: (data) {},
        builder: (context, candidateData, rejectedData) {
          if (candidateData.length == 1 && candidateData[0] >= 0) {
            _currentRemovable = candidateData[0];
          }
          return Opacity(
            opacity:
                candidateData.length == 1 && candidateData[0] >= 0 ? 1.0 : 0.5,
            child: Container(
              margin: EdgeInsets.only(bottom: 24),
              child: FloatingActionButton(
                onPressed: null,
                child: Icon(
                  Icons.delete_forever,
                  color: Colors.black,
                ),
                backgroundColor: Colors.white,
              ),
            ),
          );
        },
      ),
      floatingActionButtonLocation: FloatingActionButtonLocation.centerDocked,
    );
  }

  List<Widget> get children {
    List<Widget> subs = [];
    widget.children.forEach((basic) {});
    for (var i = 0; i < widget.children.length; i++) {
      Widget subChild;
      var basic = widget.children[i];
      if (basic is StoryTextData) {
        subChild = createTextWidget(basic);
      } else if (basic is StoryImageData) {
        subChild = createImageWidget(basic);
      } else if (basic is StorySmileyData) {
        subChild = createSmileyWidget(basic);
      }
      subChild = GestureDetector(
        child: subChild,
        onTap: widget.onBasicTap != null
            ? () {
                if (!(basic is StoryImageData)) widget.children.remove(basic);
                setState(() {});
                widget.onBasicTap(basic);
              }
            : null,
      );
      if (basic is StoryImageData) {
        subs.add(subChild);
      } else {
        Widget child = MovingWidget(
          id: i,
          child: subChild,
          canRemove: _currentRemovable == i,
          onRemove: () {},
        );
        subs.add(child);
      }
    }
    return subs;
  }

  Widget createTextWidget(StoryTextData basic) {
    return Container(
      decoration: basic.decoration.decoration,
      padding: StoryTextData.TEXT_PADDING,
      child: Text(
        basic.data,
        style: basic.decoration.style,
      ),
    );
  }

  Widget createImageWidget(StoryImageData basic) {
    return Stack(
      fit: StackFit.expand,
      children: <Widget>[
        widget.isAdvanced
            ? Image.file(
                File(
                  basic.imagePath,
                ),
                fit: BoxFit.fill,
              )
            : Container(
                width: 0,
                height: 0,
              ),
        widget.isAdvanced
            ? BackdropFilter(
                filter: ImageFilter.blur(
                  sigmaX: 60.0,
                  sigmaY: 60.0,
                ),
                child: new Container(
                    decoration: new BoxDecoration(
                        color: Colors.black.withOpacity(0.1))),
              )
            : Container(
                width: 0,
                height: 0,
              ),
        Center(
          child: Image.file(
            File(
              basic.imagePath,
            ),
            fit: BoxFit.fitWidth,
          ),
        ),
      ],
    );
  }

  Widget createSmileyWidget(StorySmileyData basic) {
    return Image.asset(
      basic.smileyPath,
      height: 100.0,
      width: 100.0,
    );
  }

  Future<AppResponse<File>> capturePNG() async {
    try {
      RenderRepaintBoundary boundary =
          (_globalKey.currentContext.findRenderObject());

      ui.Image image = await boundary.toImage(pixelRatio: 3.0);
      ByteData byteData =
          await image.toByteData(format: ui.ImageByteFormat.png);
      var pngBytes = byteData.buffer.asUint8List();
      final directory = await getTemporaryDirectory();
      final String dirPath = '${directory.path}/Pictures';
      final String filePath = '$dirPath/${timestamp()}.png';
      File file = File(filePath);
      await file.create(recursive: true);
      await file.writeAsBytes(pngBytes);
      return AppResponse.named(data: file);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  String timestamp() => DateTime.now().millisecondsSinceEpoch.toString();
}

class StoryCreateController {
  _StoryCreateWidgetState _state;

  StoryCreateController._(this._state);

  Future<AppResponse<File>> createStory() {
    return _state.capturePNG();
  }
}
